#ifndef TREESET_H
#define TREESET_H

#include "Container.h"

// Work around compilers not supporting C++11.
#define override

namespace container {

class Iterator;

class TreeSetKey: public Key {
public:
  virtual int CompareTo(Key *aOther) override;

  TreeSetKey(char *aKey) : mKey(aKey) {}
  virtual ~TreeSetKey();
private:
  char *mKey;
};

// A null-terminated linked list.
class TreeSet: public Container{
public:
  // Allows the iterator to access its internals.
  friend class TreeSetIterator;

  virtual Iterator *GetIterator() override;

  // Operation specific to TreeSet.
  void Insert(Key *aKey, Object *aObject);
  void Remove(Key *aKey);

  // Constructor omitted.
  virtual ~TreeSet();

private:
  struct Node {
    Key *mKey;
    Object *mItem;
    Node **mChildren;
  };

  Node *mRoot;
};

class TreeSetIterator: public Iterator {
public:
  virtual void First() override;
  virtual void Next() ;
  virtual Object* GetCurrentItem() override;
  virtual bool IsDone() override;

  TreeSetIterator(TreeSet *aContainer);
  virtual ~TreeSetIterator() override;

private:
  void Reset();
  TreeSet *mContainer;
  TreeSet::Node *mCurrent;
  TreeSet::Node **mRemaining;
};

} // namespace container

#endif
