#ifndef USERVALIDATOR_H
#define USERVALIDATOR_H

class UserProfile;

class PostalCodeValidator
{
public:
  virtual ~PostalCodeValidator() {}

  virtual bool Validate(const char *aPostalCode) = 0;
};

class AbstractUserValidator
{
public:
  virtual ~AbstractUserValidator();

  bool ValidateUser();

protected:
  UserProfile *GetUserProfile() { return mUserProfile; }

  virtual bool ValidateAgeRestriction() = 0;
  virtual bool ValidateAddress() = 0;
  virtual bool ValidatePhoneNumber() = 0;

  // The factory method.
  virtual PostalCodeValidator *CreatePostalCodeValidator() = 0;

private:
  bool ValidateName();
  bool ValidateAccountId();

  UserProfile *mUserProfile;
};

class USUserValidator: public AbstractUserValidator
{
public:
  virtual bool ValidateAgeRestriction();
  virtual bool ValidateAddress();
  virtual bool ValidatePhoneNumber();

protected:
  // The factory method.
  virtual PostalCodeValidator *CreatePostalCodeValidator();
};

class TWUserValidator: public AbstractUserValidator
{
public:
  virtual bool ValidateAgeRestriction();
  virtual bool ValidateAddress();
  virtual bool ValidatePhoneNumber();

protected:
  // The factory method.
  virtual PostalCodeValidator *CreatePostalCodeValidator();
};

#endif
